from setuptools import setup

setup(name='matmodlab',
      version='3.0',
      description='Material model development laboratory',
      classifiers=[  # Classifier list:  https://pypi.python.org/pypi?:action=list_classifiers
                   "Development Status :: 4 - Beta",
                   "Environment :: Console",
                   "Environment :: X11 Applications :: Qt",
                   "Intended Audience :: Science/Research",
                   "License :: OSI Approved :: MIT License",
                   "Natural Language :: English",
                   "Programming Language :: Fortran",
                   "Programming Language :: Python :: 2.7",
                   "Topic :: Scientific/Engineering",
                   "Topic :: Scientific/Engineering :: Mathematics",
                   "Topic :: Scientific/Engineering :: Physics",
                   "Topic :: Scientific/Engineering :: Visualization",
                  ],
      url='https://github.com/tjfulle/matmodlab',
      author='Timothy Fuller, Scot Swan',
      author_email='timothy.fuller@utah.edu,scot.swan@gmail.com',
      license='MIT',
      packages=[
                'matmodlab',
                'matmodlab.femlib',
                'matmodlab.femlib.fileio',
                'matmodlab.lib',
                'matmodlab.materials',
                'matmodlab.mmd',
                'matmodlab.utils',
                'matmodlab.utils.fortran',
                'matmodlab.utils.numerix',
                'matmodlab.utils.parchment',
                'matmodlab.viewer',
               ],

      data_files=[
                  ('matmodlab/materials/abaumats', [
                                                    "matmodlab/materials/abaumats/neohooke.f90",
                                                    "matmodlab/materials/abaumats/polyhyper.f90",
                                                    "matmodlab/materials/abaumats/thermoelastic.f90",
                                                    "matmodlab/materials/abaumats/uanisohyper_inv.f",
                                                    "matmodlab/materials/abaumats/uhyper.f90",
                                                   ]),
                  ('matmodlab/materials/src', [
                                               "matmodlab/materials/src/aba_io.f90",
                                               "matmodlab/materials/src/damage.f90",
                                               "matmodlab/materials/src/elastic.f90",
                                               "matmodlab/materials/src/elastic.pyf",
                                               "matmodlab/materials/src/expansion.f90",
                                               "matmodlab/materials/src/expansion.pyf",
                                               "matmodlab/materials/src/min_io.f90",
                                               "matmodlab/materials/src/mooney_rivlin.f90",
                                               "matmodlab/materials/src/mooney_rivlin.pyf",
                                               "matmodlab/materials/src/mullins.f90",
                                               "matmodlab/materials/src/plastic.f90",
                                               "matmodlab/materials/src/plastic.pyf",
                                               "matmodlab/materials/src/sdvini.f90",
                                               "matmodlab/materials/src/tensalg.f90",
                                               "matmodlab/materials/src/uanisohyper_inv_jac.f90",
                                               "matmodlab/materials/src/uanisohyper_inv.pyf",
                                               "matmodlab/materials/src/uhyper_jac.f90",
                                               "matmodlab/materials/src/uhyper.pyf",
                                               "matmodlab/materials/src/umat.pyf",
                                               "matmodlab/materials/src/visco.f90",
                                               "matmodlab/materials/src/visco.pyf",
                                              ]),
                  ('matmodlab/tests', [
                                       "matmodlab/tests/base.diff",
                                       "matmodlab/tests/cleanup",
                                       "matmodlab/tests/data_steps.base_exo",
                                       "matmodlab/tests/gen_steps.base_dbx",
                                       "matmodlab/tests/j2_plast.base_dat",
                                       "matmodlab/tests/j2_plast_iso_hard.base_dat",
                                       "matmodlab/tests/j2_plast_kin_hard.base_dat",
                                       "matmodlab/tests/j2_plast_mix_hard.base_dat",
                                       "matmodlab/tests/lin_druck_prag_spher.base_dat",
                                       "matmodlab/tests/multi_step.base_dbx",
                                       "matmodlab/tests/opt.base_dat",
                                       "matmodlab/tests/path_cycle.base_exo",
                                       "matmodlab/tests/test_abaqus_interface.py",
                                       "matmodlab/tests/test_addon_models.py",
                                       "matmodlab/tests/test_brannon_leela.py",
                                       "matmodlab/tests/testconf.py",
                                       "matmodlab/tests/test_elastic_material.py",
                                       "matmodlab/tests/test_j2_plasticity.py",
                                       "matmodlab/tests/test_linear_drucker_prager.py",
                                       "matmodlab/tests/test_mml_features.py",
                                       "matmodlab/tests/uhyper.base_dbx",
                                      ]),
                  ('matmodlab/viewer/icon', [
                                             "matmodlab/viewer/icon/atom.png",
                                             "matmodlab/viewer/icon/box.png",
                                             "matmodlab/viewer/icon/camera.png",
                                             "matmodlab/viewer/icon/field.png",
                                             "matmodlab/viewer/icon/file.png",
                                             "matmodlab/viewer/icon/folder.png",
                                             "matmodlab/viewer/icon/frame-first.png",
                                             "matmodlab/viewer/icon/frame-last.png",
                                             "matmodlab/viewer/icon/frame-next.png",
                                             "matmodlab/viewer/icon/frame-previous.png",
                                             "matmodlab/viewer/icon/graph.png",
                                             "matmodlab/viewer/icon/hidden.png",
                                             "matmodlab/viewer/icon/legend.png",
                                             "matmodlab/viewer/icon/opt.png",
                                             "matmodlab/viewer/icon/permutate.png",
                                             "matmodlab/viewer/icon/plot.png",
                                             "matmodlab/viewer/icon/plot-scale.png",
                                             "matmodlab/viewer/icon/random.png",
                                            ]),
                  ('matmodlab/utils/fortran', [
                                               "matmodlab/utils/fortran/blas_lapack-lite.f",
                                               "matmodlab/utils/fortran/dgpadm.f",
                                               "matmodlab/utils/fortran/dsyev3.f",
                                               "matmodlab/utils/fortran/fio.f90",
                                               "matmodlab/utils/fortran/MMD.f90",
                                               "matmodlab/utils/fortran/mmlabpack.f90",
                                               "matmodlab/utils/fortran/mmlfio.f90",
                                              ])
                 ],
      scripts=[
               'matmodlab/bin/fdiff',
               'matmodlab/bin/fdump',
               'matmodlab/bin/mmd',
               'matmodlab/bin/mml',
               'matmodlab/bin/mml-osx',
              ],
      install_requires=['numpy', 'scipy'],
      zip_safe=False)
